using System;
using System.Threading;
using System.Reflection;
using jabber.client;
using jabber.protocol.client;
using Excel;

class MainClass  {
	JabberClient jabber;
	Application excel;
	_Worksheet sheet;
	Range sumCell;
	private int currentRow = 1;
	private ManualResetEvent die = new ManualResetEvent(false);

	private void newMessage(object sender, Message msg) {
		if (msg.Body == "Quit") {
			die.Set();
			return;
		}
		double dValue = 0.0;
		try {
			dValue = Double.Parse(msg.Body);
		} catch (Exception ex) {
			Console.WriteLine(ex.StackTrace);
			return;
		}
		double value = UpdateExcel(msg.From, dValue);
		UpdatePresence(value);
	}
	bool connected = false;
	private void newIQ(object sender, IQ iq) {
		if ((!connected) && jabber.IsAuthenticated) {
 		  connected = true;
		  UpdatePresence(0.0);
		}
	}
	private void newPresence(object sender, Presence pres) {
		if (pres.Type == PresenceType.subscribe) {
		  Presence response = new Presence(jabber.Document);
		  response.Type = PresenceType.subscribed;
		  response.To = pres.From;
		  jabber.Write(response);
		}
	}
	public MainClass() {
		excel = InitializeExcel();
		jabber = InitializeJabber();
		die.WaitOne();
		jabber.Close();
		excel.Quit();
	}

	private Application InitializeExcel() {
		Application app = new Application();
		app.Visible = true;
		app.DisplayAlerts = false;
		Workbooks workbooks = app.Workbooks;
		_Workbook workbook = workbooks.Add(
				XlWBATemplate.xlWBATWorksheet);
		Sheets sheets = workbook.Worksheets;
		sheet = (_Worksheet)sheets[1];
		sumCell = sheet.get_Range("C1", Missing.Value);
		sumCell.Formula = "=SUM(B:B)";
		return app;
	}

	private double UpdateExcel(String who, double value) {
		Range range = sheet.get_Range(
				"A"+currentRow, "B"+currentRow);
		object [] aRow = new object[2];
		aRow[0] = who;
		aRow[1] = value;
		range.Value2 = aRow;
		currentRow++;
		return (double)sumCell.Value2;
	}
	private JabberClient InitializeJabber() {
		JabberClient c = new JabberClient();
		c.Server = "my-jabber";
		c.User = "excel";
		c.Password = "excel";
		c.Resource = "Work";
		c.AutoLogin = true;
		c.Connect();
		c.OnMessage += new MessageHandler(newMessage);
		c.OnIQ += new IQHandler(newIQ);
		c.OnPresence += new PresenceHandler(newPresence);
		return c;
	}
	private void UpdatePresence(double val) {
		jabber.Presence(PresenceType.available, "Sum="+val, 
				"Available", 0);
	}
	public static void Main(string[] args)	{
		new MainClass();
	}
}
