use strict;
use Net::Jabber 'Client';
use Tk;
use Switch;

my $jid = 'user1@my-jabber/ad';
my $pass = 'user1';
my $otherJid = 'user2@my-jabber/ad';
my $text;
my $chatEntry = '';
my $b;
my $connection;

sub buttonPress;
sub sendMsg;

sub make_gui {
  my $main = MainWindow->new(-title => "Jabber Chat");

  my $button_frame = $main->Frame();
  $button_frame->pack(-side => "bottom");
  my $bframe = $main->Frame();
  $bframe->pack(-side => "bottom");
  my $tframe = $main->Frame();
  $tframe->pack(-side => "top");

  my $lframe = $button_frame->Frame();
  $lframe->pack(-side => "left");
  my $lab = $lframe->Label(-text => "Your JID");
  $lab->pack(-side => "left");
  my $jid_entry = $lframe->Entry(-textvariable => \$jid);
  $jid_entry->pack(-side => "right");

  my $lframe2 = $button_frame->Frame();
  $lframe2->pack(-side => "left");
  my $lab = $lframe2->Label(-text => "Password ");
  $lab->pack(-side => "left");
  my $jid_entry = $lframe2->Entry(-textvariable => \$pass);
  $jid_entry->pack(-side => "right");

  my $lframe3 = $button_frame->Frame();
  $lframe3->pack(-side => "left");
  my $lab = $lframe3->Label(-text => "Other JID");
  $lab->pack(-side => "left");
  my $jid_entry = $lframe3->Entry(-textvariable => \$otherJid);
  $jid_entry->pack(-side => "right");

  $b = $button_frame->Button(-text => 'Connect', -command => \&buttonPress);
  $b->pack(-side => 'right');


  my $scroll = $tframe->Scrollbar();
  $text = $tframe->Text(-width => '80', 
          -height => '20',
          -yscrollcommand => ['set', $scroll]);
  $scroll->configure(-command => ['yview', $text]);
  $scroll->pack(-side => 'left', -fill => 'y');
  $text->pack(-side => 'top', -fill => 'both', -anchor => 'n', 
              -expand => 'true');
  $text->tag('configure', 'mine', -foreground => 'red');
  $text->tag('configure', 'his', -foreground => 'blue');

  my $entry = $bframe->Entry(-width => '80', -textvariable => \$chatEntry);
  $entry->pack(-side => 'bottom', -fill => 'both', -anchor => 's');
  $entry->bind('<Return>' => \&sendMsg);
  
}

sub buttonPress {
  my $label = $b->cget(-text);
  if ($label eq "Connect") {
    connectToJabber();
    $b->configure(-text => "Send\nPresence");
  } else {
    sendPresence(show => "chat", status => "Let's roll");
  }
}

sub connectToJabber() {
  my $uname;
  my $server;
  my $resource;
  ($uname, $server, $resource) = ($jid =~/([^@]*)@([^\/]*)\/(.*)/);
  $connection = new Net::Jabber::Client();

  $connection->SetCallBacks(message=>\&messageCB,
                            presence=>\&presenceCB);

  my $status = $connection->Connect(hostname=>$server);
  
  my @result = $connection->AuthSend(username=>$uname,
                                     password=>$pass,
                                     resource=>$resource);

  if ($result[0] ne "ok")
  {
    print "ERROR: Authorization failed: $result[0] - $result[1]\n";
    exit(0);
  }

  $connection->RosterGet();
  $connection->PresenceSend();
  
  # Look for jabber messages every 1000 millisecs
  my $id = Tk::After->new($text, 1000, 'repeat', \&pollJabber);
}


sub sendMsg {
  $connection->MessageSend(to=>$otherJid, subject=>"chat_demo_subject",
                           thread=>"chat_demo_thread",type=>"chat",
                           body=>$chatEntry);
  my $uname;
  my $server;
  my $resource;
  ($uname, $server, $resource) = ($jid =~/([^@]*)@([^\/]*)\/(.*)/);
  $text->insert("end", "<$uname> $chatEntry\n", 'mine');
  $chatEntry = '';
}


sub sendPresence {
  my %args;
  while($#_ >= 0) { $args{ lc pop(@_) } = pop(@_); }
  $args{from} = $jid;

  $connection->PresenceSend(%args);
}

sub presenceCB {
  my $sid = shift;
  my $presence = shift;

  my $from = $presence->GetFrom();
  my $type = $presence->GetType();
  switch ($type) {
    case "subscribe" {
            sendPresence(to => $from, type => "subscribed"); 
            sendPresence(to => $from, type => "subscribe");}
    case "unsubscribe" {
            sendPresence(to => $from, type => "unsubscribed"); 
            sendPresence(to => $from, type => "unsubscribe");}
  }
  my $show = $presence->GetShow();
  my $status = $presence->GetStatus();
  if ($show or $status) {
    $show = "normal" unless $show;
    $text->insert("end", "<roster> $from is now $show/$status\n");
  }
}

sub messageCB {
  my $sid = shift;
  my $msg = shift;

  my $src = $msg->GetFrom("jid")->GetUserID();
  my $msgtxt = $msg->GetBody();
  $text->insert("end", "<$src> $msgtxt\n", 'his');
}

sub pollJabber {
  my $res = 1;
  while ($res) {$res = $connection->Process(0);}
}

make_gui();
MainLoop;
